==> m4e_sdk-0.0.3-dev.14.tar.gz: lib/src/data/services/m4e_wallet_service.dart <==
import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../m4e_sdk_initialize.dart';
import '../../models/models.dart';
import '../../m4e_forms/m4e_forms.dart';
import '../api/apis.dart';

/// Exposes SDK Wallet Api methods
///
/// Accessing the instantiated `m4eWallet` methods
/// is done by:
///
/// Instantiating as a regular object
///
/// ```
/// final _m4eWallet = M4eWallet();
///
/// await _m4eWallet.getUserWallets();
/// ```
///
class M4eWalletService {
  final WalletApi _walletApi;
  final PermissionApi _permissionApi;
  final DataConnectionChecker _connectionChecker;

  M4eWalletService({
    WalletApi walletApi,
    PermissionApi permissionApi,
    DataConnectionChecker connectionChecker,
  })  : _walletApi = walletApi ?? m4eSDK<WalletApi>(),
        _permissionApi = permissionApi ?? m4eSDK<PermissionApi>(),
        _connectionChecker =
            connectionChecker ?? m4eSDK<DataConnectionChecker>();

  /// runs search query after creating it
  Future<M4eWalletList> _runSearchQueryForM4eWallets(
      M4eUniqueId queryId) async {
    try {
      return await _walletApi.runSearchQuery(queryId);
    } catch (e) {
      rethrow;
    }
  }

  /// Returns current authenticated m4e user `Wallets` from m4e when no [otherUserId] is provided
  ///
  /// Returns other m4e user `Wallets` which the current authenticated user has [SEE, PAY] permissions on from m4e
  ///
  /// Example:
  ///
  /// ```
  /// final service = M4eWalletService(); // instantiate M4eWalletService
  ///
  /// final currentUserWallets = service.getWallets(currentUserId: 97fdebd0-7424-474e-a17c-67be85cc5638); // returns 97fdebd0-7424-474e-a17c-67be85cc5638 wallets with [VIEW, GRANT, ALTER] permissions
  /// ```
  ///
  /// ```
  /// final otherUserWallets = service.getWallets(currentUserId: 97fdebd0-7424-474e-a17c-67be85cc5638, otherUserId: 97fdebd0-7424-474e-a17c-67be85cc5638);  // returns other-m4e-user with [M4eUniqueId] wallets which the current authenticated user has a `SEE` and 'PAY' permissions on
  /// ```
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code when device is not connected to the internet
  ///
  Future<M4eWalletList> getWallets(
      {@required M4eUniqueId currentUserId, M4eUniqueId otherUserId}) async {
    assert(
        currentUserId != null, 'getWallets [currentUserId] should not be null');

    SearchFormTypeCollection _searchFormTypeCollection;

    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _isAuthenticatedUserWalletsRequest = otherUserId == null;

        final _ownerWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
          EntityWalletAccess.view,
          EntityWalletAccess.grant,
          EntityWalletAccess.alter,
        ];

        final _otherUsersWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
          EntityWalletAccess.see,
          EntityWalletAccess.pay,
        ];

        if (_isAuthenticatedUserWalletsRequest) {
          // accessing current authenticated user wallets
          _searchFormTypeCollection = SingleClauseSearchForm(
            clause: SearchClause(
              subjectId: currentUserId,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _ownerWalletAccessPermissionsSet,
            ),
          );
        } else {
          /// accounts for cases where the user wants to fetch his wallets as recipient
          /// (in cases where transaction is maded between owner wallets)
          ///
          /// this will use simple clause query set for same user-id as `currentUserId` and `otherUserId`
          /// and compound clause query for different case
          if (currentUserId == otherUserId) {
            _searchFormTypeCollection = SingleClauseSearchForm(
              clause: SearchClause(
                subjectId: currentUserId,
                type: M4eAccessPermissionType.ENTITY_WALLET,
                encoding: _ownerWalletAccessPermissionsSet,
              ),
            );
          } else {
            /// accessing wallets owned by other user
            final _ownerWalletsAccessSearchClause = SearchClause(
              subjectId: otherUserId,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _ownerWalletAccessPermissionsSet,
            );

            /// accessing wallets current user has [SEE, PAY] ENTITY_WALLET permissions on
            final _currentUsersWalletAccessSearchClause = SearchClause(
              subjectId: currentUserId,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _otherUsersWalletAccessPermissionsSet,
            );

            _searchFormTypeCollection = MultipleClauseSearchForm(
              conjunction: M4eSearchConjunctionType.AND,
              clauses: <SearchClause>[
                _ownerWalletsAccessSearchClause,
                _currentUsersWalletAccessSearchClause,
              ],
            );
          }
        }

        final _runQueryUniqueId =
            await _walletApi.queryUserWallets(_searchFormTypeCollection);

        final _walletList =
            await _runSearchQueryForM4eWallets(_runQueryUniqueId);

        if (_walletList.wallets.isEmpty) {
          throw M4eExceptionMessages.kNotFoundException;
        }

        return _walletList;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }

  /// Creates an m4e user-wallet and returns created wallet [M4eUniqueId]
  ///
  /// Creates owner-permission on wallet
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<M4eUniqueId> createWallet(
      M4eUniqueId userId, M4eWalletForm m4eWalletForm) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _walletId = await _walletApi.createWallet(userId, m4eWalletForm);

        // create EntityWalletAccess permission form
        final _permissionForm = M4ePermissionForm(
          subId: userId,
          objId: _walletId,
          type: EntityWalletAccess.TYPE,
          encoding: <M4eAccessPermissionSet>[
            EntityWalletAccess.see,
            EntityWalletAccess.view,
            EntityWalletAccess.pay,
            EntityWalletAccess.grant,
            EntityWalletAccess.alter,
            EntityWalletAccess.create,
            EntityWalletAccess.destroy,
          ],
        );

        // set owner permissions on wallet
        await _permissionApi.setPermission(_permissionForm);

        return _walletId;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }
}

==> m4e_sdk-0.0.4-dev.16.tar.gz: lib/src/data/services/m4e_wallet_service.dart <==
import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../m4e_sdk_initialize.dart';
import '../../models/models.dart';
import '../../m4e_forms/m4e_forms.dart';
import '../api/apis.dart';

/// Exposes SDK Wallet Api methods
///
/// Accessing the instantiated `m4eWallet` methods
/// is done by:
///
/// Instantiating as a regular object
///
/// ```
/// final _m4eWallet = M4eWallet();
///
/// await _m4eWallet.getUserWallets();
/// ```
///
class M4eWalletService {
  final UserApi _userApi;
  final WalletApi _walletApi;
  final PermissionApi _permissionApi;
  final DataConnectionChecker _connectionChecker;

  M4eWalletService({
    UserApi userApi,
    WalletApi walletApi,
    PermissionApi permissionApi,
    DataConnectionChecker connectionChecker,
  })  : _userApi = userApi ?? m4eSDK<UserApi>(),
        _walletApi = walletApi ?? m4eSDK<WalletApi>(),
        _permissionApi = permissionApi ?? m4eSDK<PermissionApi>(),
        _connectionChecker =
            connectionChecker ?? m4eSDK<DataConnectionChecker>();

  /// query for user with phone number
  Future<M4eUser> _getM4eUserWithPhoneNumber(M4ePhoneNumber phoneNumber) async {
    try {
      return await _userApi.getM4eUser(phoneNumber);
    } catch (e) {
      rethrow;
    }
  }

  /// runs search query after creating it
  Future<M4eWalletList> _runSearchQueryForM4eWallets(
      M4eUniqueId queryId) async {
    try {
      return await _walletApi.runSearchQuery(queryId);
    } catch (e) {
      rethrow;
    }
  }

  /// Returns current authenticated m4e user `Wallets` from m4e when no [otherUserId] is provided
  ///
  /// Returns other m4e user `Wallets` which the current authenticated user has [SEE, PAY] permissions on from m4e
  ///
  /// Example:
  ///
  /// ```
  /// final service = M4eWalletService(); // instantiate M4eWalletService
  ///
  /// final currentUserWallets = service.getWalletsWithPhoneNumber(currentUserId: 97fdebd0-7424-474e-a17c-67be85cc5638); // returns 97fdebd0-7424-474e-a17c-67be85cc5638 wallets with [VIEW, GRANT, ALTER] permissions
  /// ```
  ///
  /// ```
  /// final otherUserWallets = service.getWalletsByPhoneNumber(currentUserId: 97fdebd0-7424-474e-a17c-67be85cc5638, otherUserPhoneNumber: M4ePhoneNumber(country: 'GHANA', number: '0541234567'));  // returns other-m4e-user with [M4ePhoneNumber] wallets which the current authenticated user has a `SEE` and 'PAY' permissions on
  /// ```
  ///
  /// Throws AuthException with code NO_AUTH_USER when no user is authenticated
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code when device is not connected to the internet
  ///
  Future<M4eWalletList> getWalletsByPhoneNumber({
    @required M4eUniqueId currentUserId,
    M4ePhoneNumber otherUserPhoneNumber,
  }) async {
    assert(currentUserId != null,
        'getWallets [getWalletsWithPhoneNumber] should not be null');

    M4eUser _otherM4eUserDetails;
    SearchFormTypeCollection _searchFormTypeCollection;

    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        if (otherUserPhoneNumber != null) {
          /// gets m4eUser with phone number if phone number is passed in
          _otherM4eUserDetails =
              await _getM4eUserWithPhoneNumber(otherUserPhoneNumber);
        }

        final _isAuthenticatedUserWalletsRequest = otherUserPhoneNumber == null;

        final _ownerWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
          EntityWalletAccess.view,
          EntityWalletAccess.grant,
          EntityWalletAccess.alter,
        ];

        final _otherUsersWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
          EntityWalletAccess.see,
          EntityWalletAccess.pay,
        ];

        if (_isAuthenticatedUserWalletsRequest) {
          // accessing current authenticated user wallets
          _searchFormTypeCollection = SingleClauseSearchForm(
            clause: SearchClause(
              subjectId: currentUserId,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _ownerWalletAccessPermissionsSet,
            ),
          );
        } else {
          /// accounts for cases where the user wants to fetch his wallets as recipient
          /// (in cases where transaction is maded between owner wallets)
          ///
          /// this will use simple clause query set for same user-id as `currentUserId` and `otherUserId`
          /// and compound clause query for different case
          if (currentUserId == _otherM4eUserDetails.id) {
            _searchFormTypeCollection = SingleClauseSearchForm(
              clause: SearchClause(
                subjectId: currentUserId,
                type: M4eAccessPermissionType.ENTITY_WALLET,
                encoding: _ownerWalletAccessPermissionsSet,
              ),
            );
          } else {
            /// accessing wallets owned by other user
            final _ownerWalletsAccessSearchClause = SearchClause(
              subjectId: _otherM4eUserDetails.id,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _ownerWalletAccessPermissionsSet,
            );

            /// accessing wallets current user has [SEE, PAY] ENTITY_WALLET permissions on
            final _currentUsersWalletAccessSearchClause = SearchClause(
              subjectId: currentUserId,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _otherUsersWalletAccessPermissionsSet,
            );

            _searchFormTypeCollection = MultipleClauseSearchForm(
              conjunction: M4eSearchConjunctionType.AND,
              clauses: <SearchClause>[
                _ownerWalletsAccessSearchClause,
                _currentUsersWalletAccessSearchClause,
              ],
            );
          }
        }

        final _runQueryUniqueId =
            await _walletApi.queryUserWallets(_searchFormTypeCollection);

        final _walletList =
            await _runSearchQueryForM4eWallets(_runQueryUniqueId);

        if (_walletList.wallets.isEmpty) {
          throw M4eExceptionMessages.kNotFoundException;
        }

        return _walletList;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }

  /// Returns other m4e user `Wallets` which the current authenticated user has [SEE, PAY] permissions on from m4e
  ///
  /// Example:
  ///
  /// ```
  /// final service = M4eWalletService(); // instantiate M4eWalletService
  ///
  ///
  /// final otherUserWallets = service.getWalletsByUserId(currentUserId: 97fdebd0-7424-474e-a17c-67be85cc5638, otherUserId: 97fdebd0-7424-474e-a17c-67be85cc5638);  // returns other-m4e-user with [M4ePhoneNumber] wallets which the current authenticated user has a `SEE` and 'PAY' permissions on
  /// ```
  ///
  /// Throws AuthException with code NO_AUTH_USER when no user is authenticated
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code when device is not connected to the internet
  ///
  Future<M4eWalletList> getWalletsByUserId({
    @required M4eUniqueId currentUserId,
    @required M4eUniqueId otherUserId,
  }) async {
    assert(currentUserId != null,
        'getWalletsWithUserId [currentUserId] should not be null');
    assert(otherUserId != null,
        'getWalletsWithUserId [otherUserId] should not be null');

    SearchFormTypeCollection _searchFormTypeCollection;

    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _ownerWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
          EntityWalletAccess.view,
          EntityWalletAccess.grant,
          EntityWalletAccess.alter,
        ];

        final _otherUsersWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
          EntityWalletAccess.see,
          EntityWalletAccess.pay,
        ];

        /// accounts for cases where the user wants to fetch his wallets as recipient
        /// (in cases where transaction is maded between owner wallets)
        ///
        /// this will use simple clause query set for same user-id as `currentUserId` and `otherUserId`
        /// and compound clause query for different case
        if (currentUserId == otherUserId) {
          _searchFormTypeCollection = SingleClauseSearchForm(
            clause: SearchClause(
              subjectId: currentUserId,
              type: M4eAccessPermissionType.ENTITY_WALLET,
              encoding: _ownerWalletAccessPermissionsSet,
            ),
          );
        } else {
          /// accessing wallets owned by other user
          final _ownerWalletsAccessSearchClause = SearchClause(
            subjectId: otherUserId,
            type: M4eAccessPermissionType.ENTITY_WALLET,
            encoding: _ownerWalletAccessPermissionsSet,
          );

          /// accessing wallets current user has [SEE, PAY] ENTITY_WALLET permissions on
          final _currentUsersWalletAccessSearchClause = SearchClause(
            subjectId: currentUserId,
            type: M4eAccessPermissionType.ENTITY_WALLET,
            encoding: _otherUsersWalletAccessPermissionsSet,
          );

          _searchFormTypeCollection = MultipleClauseSearchForm(
            conjunction: M4eSearchConjunctionType.AND,
            clauses: <SearchClause>[
              _ownerWalletsAccessSearchClause,
              _currentUsersWalletAccessSearchClause,
            ],
          );
        }

        final _runQueryUniqueId =
            await _walletApi.queryUserWallets(_searchFormTypeCollection);

        final _walletList =
            await _runSearchQueryForM4eWallets(_runQueryUniqueId);

        if (_walletList.wallets.isEmpty) {
          throw M4eExceptionMessages.kNotFoundException;
        }

        return _walletList;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }

  /// Returns wallet balance for specified wallet-id
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<M4eBalance> getWalletBalance(M4eUniqueId walletId) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _balance = await _walletApi.getWalletBalance(walletId);

        return _balance;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }

  /// Creates an m4e user-wallet and returns created wallet [M4eUniqueId]
  ///
  /// Creates owner-permission on wallet
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<M4eUniqueId> createWallet(
      M4eUniqueId userId, M4eWalletForm m4eWalletForm) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _walletId = await _walletApi.createWallet(userId, m4eWalletForm);

        // create EntityWalletAccess permission form
        final _permissionForm = M4ePermissionForm(
          subId: userId,
          objId: _walletId,
          type: EntityWalletAccess.TYPE,
          encoding: <M4eAccessPermissionSet>[
            EntityWalletAccess.see,
            EntityWalletAccess.view,
            EntityWalletAccess.pay,
            EntityWalletAccess.grant,
            EntityWalletAccess.alter,
            EntityWalletAccess.create,
            EntityWalletAccess.destroy,
          ],
        );

        // set owner permissions on wallet
        await _permissionApi.setPermission(_permissionForm);

        return _walletId;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }
}
